from .command import CmdOpt, CommandMaster
from .common import Version, get_pkg_version
from .config import APP_ENV, Config, Env
from .convert import safe_float, safe_int
from .dict import merge_dicts
from .fetch import async_fetch, fetch
from .file import MFile
from .funny import advance_call, advance_class_call, async_advance_call
from .list import (
    advance_split,
    drop_empty_from_list,
    json_list_stringify_limit,
    quick_value,
    split_and_drop_empty,
)
from .mxios import AioResponse, Mxios, Response
from .retry import async_retry, retry
from .url import (
    drop_url_anchor,
    gen_proxy_url,
    get_url_domain,
    get_url_params,
    get_url_path,
    is_relative_url,
)



import sys
from enum import Enum
from pathlib import Path
from typing import List, Optional, Type, TypeVar, Union, cast

import yaml
from pydantic import BaseModel

from .common import Version, get_pkg_version
from .exceptions import ConfigStructureError

__pydantic_version__: Version = Version(get_pkg_version("pydantic"))
_config_root: Path = Path("./configs")


class Env(Enum):
    """预设环境变量"""

    Local = "local"
    Dev = "dev"
    Test = "test"
    Prod = "prod"
    Custom = "custom"
    Default = "dev"  # noqa: PIE796


APP_ENV: str = Env.Default.value
Self = TypeVar("Self", bound="Config")

class Config(BaseModel):
    """配置基类"""

    @classmethod
    def set_configs_root(cls, root: Union[str, Path]):
        """设置配置文件根目录"""
        global _config_root
        _config_root = Path(root)

    @classmethod
    def load_config(cls: Type[Self], create_if_not_exists: bool = True) -> Self:
        """加载配置

        :param create_if_not_exists: 如果配置文件不存在，是否创建

        :raises Exception: 配置文件格式错误
        """

        global APP_ENV, _config_root
        config_path = _config_root / f"config.{APP_ENV}.yaml"
        config_path.parent.mkdir(parents=True, exist_ok=True)
        try:
            with config_path.open("r", encoding="utf-8") as file:
                try:
                    if __pydantic_version__ < Version("2.0.0"):  # 兼容旧版本的 pydantic
                        obj = cls.parse_obj(yaml.safe_load(file))
                    else:
                        obj = cls.model_validate(yaml.safe_load(file))
                except yaml.YAMLError as e:
                    raise ConfigStructureError("配置文件格式错误") from e
        except Exception:
            if create_if_not_exists:
                cls.dump_config_template()
                return cls.load_config(False)
            raise
        else:
            return cast(Self, obj)

    @classmethod
    def dump_modal(cls):
        """导出配置模板

        :return: 配置模板
        """
        return cls().__dict__

    @classmethod
    def dump_config_template(cls, envs: Optional[List[str]] = None):
        """导出配置模板"""

        global _config_root
        if envs is None:
            envs = [APP_ENV]
        for env in envs:
            config_path = _config_root / f"config.{env}.yaml"
            config_path.parent.mkdir(parents=True, exist_ok=True)
            config_path.write_text(
                yaml.dump(
                    cls.dump_modal(),
                    allow_unicode=True,
                    sort_keys=False,
                    default_flow_style=False,
                ),
                encoding="utf-8",
            )
        return cls.dump_modal()

    def dump_config(self, envs: Optional[List[str]] = None):
        """导出配置"""

        global _config_root
        if envs is None:
            envs = [APP_ENV]
        for env in envs:
            config_path = _config_root / f"config.{env}.yaml"
            config_path.parent.mkdir(parents=True, exist_ok=True)
            if config_path.exists():
                # 如果配置文件存在，则只补充缺失的配置项
                try:
                    origin_config = yaml.safe_load(
                        config_path.read_text(encoding="utf-8"),
                    )
                except yaml.YAMLError as e:
                    raise ConfigStructureError("配置文件格式错误") from e
                for key, value in self.__dict__.items():
                    if key not in origin_config:
                        origin_config[key] = value
                with config_path.open("w", encoding="utf-8") as f:
                    yaml.dump(
                        origin_config,
                        f,
                        default_flow_style=False,
                        sort_keys=False,
                        encoding="utf-8",
                        allow_unicode=True,
                    )
            else:
                with config_path.open("w", encoding="utf-8") as f:
                    yaml.dump(
                        self.model_dump(),
                        f,
                        default_flow_style=False,
                        sort_keys=False,
                        encoding="utf-8",
                        allow_unicode=True,
                    )

    def reload_config(self: Self) -> Self:
        """重新加载配置"""
        obj: Self = self.load_config()
        self.__dict__.update(obj.__dict__)
        return obj

    def gen_config_schema(self):
        """生成配置结构

        :return: 配置结构
        """
        if __pydantic_version__ < Version("2.0.0"):
            return self.schema()["properties"]
        return self.__class__.model_json_schema()["properties"]


for arg in sys.argv[1:]:
    if arg.startswith("env="):
        target_env = arg.split("=")[-1]
        for env in Env:
            if env.value.lower() == target_env.lower():
                APP_ENV = env.value
                break
        else:
            raise Exception(f'环境变量 "{target_env}" 不存在')
        break
else:
    APP_ENV = Env.Default.value



from urllib.parse import quote, unquote, urlparse


def get_url_domain(url: str) -> str:
    """获取url的域名"""

    try:
        return url.split("//")[1].split("/")[0]
    except Exception:
        return ""


def get_url_path(url: str) -> str:
    """获取url的路径"""

    path = url.split("//")[1].split("/", 1)  # Split only once
    if len(path) > 1:
        return "/" + path[1]
    return "/"


def get_url_params(url: str, use_unquote: bool = True) -> dict:
    """获取url的参数"""

    try:
        params = {}
        for param in url.split("?")[1].split("&"):
            k, v = param.split("=")
            params[k] = v if not use_unquote else unquote(v)
    except Exception:
        return {}
    else:
        return params


def drop_url_anchor(url: str) -> str:
    """去除url中的锚点"""

    if url.startswith("#"):
        return ""
    return url.split("#")[0]


def is_relative_url(url: str) -> bool:
    """判断url是否为相对路径"""

    return url.startswith(("/", "#"))


def trans_str_to_url(s: str) -> str:
    """将字符串转换为url"""

    return quote(s, safe="")


def gen_proxy_url(
    proxy_host: str,
    proxy_port: int,
    proxy_username: str = "",
    proxy_password: str = "",
    protocol: str = "http",
) -> str:
    return urlparse(
        f"{protocol}//{proxy_username}:{proxy_password}@{proxy_host}:{proxy_port}",
    ).geturl()



from typing import Any, Dict, List, Optional, Union

import aiohttp
import requests

from .fetch import async_fetch, fetch, parse_cookies

try:
    import ujson as json  # type: ignore
except ImportError:
    import json


import json


class Response:
    """响应对象"""

    def __init__(
        self,
        text: str,
        status_code: int,
        raw_resp: requests.Response,
    ):
        self.text: str = text
        self.status_code: int = status_code
        self._raw_resp: requests.Response = raw_resp

    @property
    def raw_resp(self) -> requests.Response:
        """原始响应对象"""
        return self._raw_resp

    def __str__(self) -> str:
        return f"Response(status_code={self.status_code}, content={self.text})"

    def json(self) -> Any:
        return json.loads(self.text)


class AioResponse(Response):
    """异步响应对象"""

    def __init__(
        self,
        text: str,
        status_code: int,
        raw_resp: aiohttp.ClientResponse,
    ):
        self.text: str = text
        self.status_code: int = status_code
        self._raw_resp: aiohttp.ClientResponse = raw_resp

    @property
    def raw_resp(self) -> aiohttp.ClientResponse:
        """原始响应对象"""
        return self._raw_resp


class Mxios:
    def __init__(
        self,
        base_url: str = "",
        headers: Optional[Dict[str, str]] = None,
        cookies: Optional[Dict[str, str]] = None,
        proxy_server: str = "",
        timeout: int = 60,
    ):
        self.base_url: str = base_url[:-1] if base_url.endswith("/") else base_url
        self.headers: Dict[str, str] = headers or {}
        self.cookies: Dict[str, str] = cookies or {}
        self.proxy_server: str = proxy_server
        self.timeout: int = timeout

    def fetch(
        self,
        method: str,
        url: str,
        params: Optional[Dict[str, str]] = None,
        headers: Optional[Dict[str, str]] = None,
        cookies: Optional[Union[Dict[str, str], str]] = None,
        data: Union[str, Dict, List] = "",
        proxy_server: str = "",
        timeout: int = 60,
        ssl_verify: bool = True,
        allow_redirects=True,
    ) -> Response:
        """发起同步请求

        Args:
            url (str): 请求地址
            params (Optional[Dict[str, str]], optional): 请求参数. Defaults to None.
            headers (Optional[Dict[str, str]], optional): 请求头. Defaults to None.
            cookies (Optional[Dict[str, str]], optional): 请求cookie. Defaults to None.
            data (Union[str, Dict, List], optional): 请求体. Defaults to "{}".
            proxy_server (str, optional): 代理服务器. Defaults to "".
            timeout (int, optional): 超时时间. Defaults to 60.
            ssl_verify (bool, optional): 是否验证ssl. Defaults to True.
            allow_redirects (bool, optional): 是否允许重定向. Defaults to True.
        Returns:
            Response: 返回响应对象
        """

        headers = headers or {}
        headers = {**self.headers, **headers}

        if isinstance(cookies, str):
            cookies = parse_cookies(cookies)
        cookies = cookies or {}
        cookies = {**self.cookies, **cookies}

        params = params or {}

        if isinstance(data, (dict, list)):
            data = json.dumps(data)

        if not (url.startswith(("http://", "https://"))):
            url = self.base_url + url

        resp_status_code, resp_text, raw_resp = fetch(
            url=url,
            method=method,
            params=params,
            data=data,
            headers=headers,
            proxy_server=self.proxy_server or proxy_server,
            timeout=timeout,
            ssl_verify=ssl_verify,
            allow_redirects=allow_redirects,
        )

        return Response(text=resp_text, status_code=resp_status_code, raw_resp=raw_resp)

    async def async_fetch(
        self,
        method: str,
        url: str,
        params: Optional[Dict[str, str]] = None,
        headers: Optional[Dict[str, str]] = None,
        cookies: Optional[Union[Dict[str, str], str]] = None,
        data: Union[str, Dict, List] = "",
        proxy_server: str = "",
        timeout: int = 60,
        ssl_verify: bool = True,
        allow_redirects=True,
    ) -> AioResponse:
        """发起异步请求

        Args:
            url (str): 请求地址
            params (Optional[Dict[str, str]], optional): 请求参数. Defaults to None.
            headers (Optional[Dict[str, str]], optional): 请求头. Defaults to None.
            cookies (Optional[Dict[str, str]], optional): 请求cookie. Defaults to None.
            data (Union[str, Dict, List], optional): 请求体. Defaults to "{}".
            proxy_server (str, optional): 代理服务器. Defaults to "".
            timeout (int, optional): 超时时间. Defaults to 60.
            ssl_verify (bool, optional): 是否验证ssl. Defaults to True.
            allow_redirects (bool, optional): 是否允许重定向. Defaults to True.
        Returns:
            AioResponse: 返回响应对象
        """

        headers = headers or {}
        headers = {**self.headers, **headers}

        if isinstance(cookies, str):
            cookies = parse_cookies(cookies)
        cookies = cookies or {}
        cookies = {**self.cookies, **cookies}

        params = params or {}

        if isinstance(data, (dict, list)):
            data = json.dumps(data)

        if not (url.startswith(("http://", "https://"))):
            url = self.base_url + url

        resp_status_code, resp_text, raw_resp = await async_fetch(
            url=url,
            method=method,
            params=params,
            data=data,
            headers=headers,
            proxy_server=self.proxy_server or proxy_server,
            timeout=timeout,
            ssl_verify=ssl_verify,
            allow_redirects=allow_redirects,
        )

        return AioResponse(
            text=resp_text,
            status_code=resp_status_code,
            raw_resp=raw_resp,
        )

    def get(
        self,
        url,
        params: Optional[Dict[str, str]] = None,
        headers: Optional[Dict[str, str]] = None,
        proxy_server: str = "",
        timeout: int = 60,
        ssl_verify: bool = True,
        allow_redirects=True,
    ) -> Response:
        return self.fetch(
            method="get",
            url=url,
            params=params,
            headers=headers,
            proxy_server=proxy_server,
            timeout=timeout,
            ssl_verify=ssl_verify,
            allow_redirects=allow_redirects,
        )

    def post(
        self,
        url,
        data: Union[str, Dict, List] = "",
        headers: Optional[Dict[str, str]] = None,
        cookies: Optional[Dict[str, str]] = None,
        proxy_server: str = "",
        timeout: int = 60,
        ssl_verify: bool = True,
        allow_redirects=True,
    ) -> Response:
        return self.fetch(
            method="post",
            url=url,
            data=data,
            headers=headers,
            cookies=cookies,
            proxy_server=proxy_server,
            timeout=timeout,
            ssl_verify=ssl_verify,
            allow_redirects=allow_redirects,
        )

    def put(
        self,
        url,
        params: Optional[Dict[str, str]] = None,
        data: Union[str, Dict, List] = "",
        headers: Optional[Dict[str, str]] = None,
        cookies: Optional[Dict[str, str]] = None,
        proxy_server: str = "",
        timeout: int = 60,
        ssl_verify: bool = True,
        allow_redirects=True,
    ) -> Response:
        return self.fetch(
            method="put",
            url=url,
            params=params,
            data=data,
            headers=headers,
            cookies=cookies,
            proxy_server=proxy_server,
            timeout=timeout,
            ssl_verify=ssl_verify,
            allow_redirects=allow_redirects,
        )

    def delete(
        self,
        url,
        params: Optional[Dict[str, str]] = None,
        headers: Optional[Dict[str, str]] = None,
        cookies: Optional[Dict[str, str]] = None,
        proxy_server: str = "",
        timeout: int = 60,
        ssl_verify: bool = True,
        allow_redirects=True,
    ) -> Response:
        return self.fetch(
            method="delete",
            url=url,
            params=params,
            headers=headers,
            cookies=cookies,
            proxy_server=proxy_server,
            timeout=timeout,
            ssl_verify=ssl_verify,
            allow_redirects=allow_redirects,
        )

    async def async_get(
        self,
        url,
        params: Optional[Dict[str, str]] = None,
        headers: Optional[Dict[str, str]] = None,
        proxy_server: str = "",
        timeout: int = 60,
        ssl_verify: bool = True,
        allow_redirects=True,
    ) -> AioResponse:
        return await self.async_fetch(
            method="get",
            url=url,
            params=params,
            headers=headers,
            proxy_server=proxy_server,
            timeout=timeout,
            ssl_verify=ssl_verify,
            allow_redirects=allow_redirects,
        )

    async def async_post(
        self,
        url,
        data: Union[str, Dict, List] = "",
        headers: Optional[Dict[str, str]] = None,
        cookies: Optional[Dict[str, str]] = None,
        proxy_server: str = "",
        timeout: int = 60,
        ssl_verify: bool = True,
        allow_redirects=True,
    ) -> AioResponse:
        return await self.async_fetch(
            method="post",
            url=url,
            data=data,
            headers=headers,
            cookies=cookies,
            proxy_server=proxy_server,
            timeout=timeout,
            ssl_verify=ssl_verify,
            allow_redirects=allow_redirects,
        )

    async def async_put(
        self,
        url,
        params: Optional[Dict[str, str]] = None,
        data: Union[str, Dict, List] = "",
        headers: Optional[Dict[str, str]] = None,
        cookies: Optional[Dict[str, str]] = None,
        proxy_server: str = "",
        timeout: int = 60,
        ssl_verify: bool = True,
        allow_redirects=True,
    ) -> AioResponse:
        return await self.async_fetch(
            method="put",
            url=url,
            params=params,
            data=data,
            headers=headers,
            cookies=cookies,
            proxy_server=proxy_server,
            timeout=timeout,
            ssl_verify=ssl_verify,
            allow_redirects=allow_redirects,
        )

    async def async_delete(
        self,
        url,
        params: Optional[Dict[str, str]] = None,
        headers: Optional[Dict[str, str]] = None,
        cookies: Optional[Dict[str, str]] = None,
        proxy_server: str = "",
        timeout: int = 60,
        ssl_verify: bool = True,
        allow_redirects=True,
    ) -> AioResponse:
        return await self.async_fetch(
            method="delete",
            url=url,
            params=params,
            headers=headers,
            cookies=cookies,
            proxy_server=proxy_server,
            timeout=timeout,
            ssl_verify=ssl_verify,
            allow_redirects=allow_redirects,
        )



from typing import List

try:
    import ujson as json
except ImportError:
    import json


def quick_value(ls: List, substitute: str = "") -> str:
    """快速列表取值"""
    if isinstance(ls, str):
        return ls.strip() if (ls and ls.strip()) else substitute
    if len(ls) == 0:
        return substitute
    return ls[0].strip() if (ls[0] and ls[0].strip()) else substitute

def drop_empty_from_list(lst: List) -> List[str]:
    """去除列表中的空字符串元素"""

    return [i for i in lst if i and i.strip()]

def split_and_drop_empty(s: str, sep: str) -> List[str]:
    """切割字符串并去除空字符串元素"""
    return drop_empty_from_list(s.split(sep))

def json_list_stringify_limit(lst: List, limit=1024) -> str:
    """列表转json列表长度自动限制(丢弃超出限制的元素)"""
    while True:
        if len(lst) < 1:
            return "[]"
        res = json.dumps(lst, ensure_ascii=False)
        if len(res) <= limit:
            return res
        lst = lst[:-1]

def advance_split(s: str, sep: str, filter_empty=True) -> List[str]:
    """高级切割字符串 (支持单双引号配对防截断和引号转义)"""
    res = []
    temp = ""
    in_single_quote = False
    in_double_quote = False
    escape = False
    for i in s:
        if escape:
            temp += i
            escape = False
            continue
        if i == "\\":
            escape = True
            continue
        if i == "'":
            in_single_quote = not in_single_quote
        if i == '"':
            in_double_quote = not in_double_quote
        if i == sep and not in_single_quote and not in_double_quote:
            res.append(temp)
            temp = ""
        else:
            temp += i
    res.append(temp)

    # 过滤空字符串
    if filter_empty:
        res = [i for i in res if i.strip()]

    # 去除引号
    res = [i[1:-1] if i.startswith('"') and i.endswith('"') else i for i in res]
    res = [i[1:-1] if i.startswith("'") and i.endswith("'") else i for i in res]

    return res  # noqa: RET504



from typing import Dict


def merge_dicts(*args: Dict) -> Dict:
    """合并多个字典的键值对 后面的字典会覆盖前面"""

    res = {}
    for d in args:
        for k, v in d.items():
            if d.get(k):
                res[k] = v
    return res



from typing import Any, Optional


def safe_int(s: Any, substitute: Optional[int] = None) -> Optional[int]:
    """不会报错的int转换

    :param s: 要转换的任意类型
    """
    try:
        return int(s)
    except Exception:
        return substitute

def safe_float(s: Any, substitute: Optional[float] = None) -> Optional[float]:
    """不会报错的float转换

    :param s: 要转换的任意类型
    """
    try:
        return float(s)
    except Exception:
        return substitute



import inspect
from typing import Any, Callable, Dict, List, Optional, Type, Union

from .list import advance_split

_FORBIDDEN_CHARACTERS = [
    " ",
    "\t",
    "\n",
    "\r",
    "\v",
    "\f",
    "\0",
    "\\",
    '"',
    "'",
    "=",
    ":",
    "|",
    ">",
    "<",
    "&",
    ";",
    "!",
    "@",
    "#",
    "$",
    "%",
    "^",
    "*",
    "(",
    ")",
    "+",
    "[",
    "]",
    "{",
    "}",
    "?",
    "/",
    "`",
    "~",
    ",",
    ".",
]


class CmdOpt:
    def __init__(
        self,
        full_name: str,
        short_name: str,
        info: str,
        Type: Union[Type[bool], Type[str], Type[int], Type[float]],
        default: Any = None,
    ):
        # 检查参数类型是否正确
        if default is None:
            default = False if Type == bool else None

        # 特殊字符检查
        for char in _FORBIDDEN_CHARACTERS:
            if char in full_name:
                raise ValueError(
                    f"The full name of an option should not contain '{char}'.",
                )
            if char in short_name:
                raise ValueError(
                    f"The short name of an option should not contain '{char}'.",
                )

        self.full_name = full_name
        self.short_name = short_name
        self.info = info
        self.Type = Type
        self.default = default

    def match(self, s: str) -> bool:
        """判断字符串是否匹配选项

        Args:
            s (str): 字符串

        Returns:
            bool: 是否匹配
        """

        return s == f"--{self.full_name}" or s == f"-{self.short_name}"

    def has_value(self) -> bool:
        """判断选项是否需要值

        Returns:
            bool: 是否需要值
        """

        return self.Type != bool

    def gen_format(self, short: bool = False) -> str:
        """生成选项格式化字符串

        Returns:
            str: 选项格式化字符串
        """

        if short:
            return (
                f"[-{self.short_name}]"
                if self.Type == bool
                else f"[-{self.short_name} <{self.short_name}>]"
            )
        return (
            f"[--{self.full_name}]"
            if self.Type == bool
            else f"[--{self.full_name} <{self.full_name}>]"
        )


class _CommandParam:
    def __init__(
        self,
        name: str,
        Type: Union[Type[str], Type[int], Type[float]],
        default: Any,
    ):
        self.name = name
        self.Type = Type
        self.default = default

    def gen_format(self) -> str:
        """生成参数格式化字符串

        Returns:
            str: 参数格式化字符串
        """

        if self.Type == bool:
            return f"[{self.name}]"
        return f"<{self.name}>"


def _get_func_param_info(func) -> List[_CommandParam]:
    """获取函数的参数信息

    Args:
        func (Callable): 函数

    Returns:
        List[_CommandParam]: 函数的参数对象列表
    """

    sig = inspect.signature(func)
    params = sig.parameters

    param_info = []
    for param in params.values():
        if param.name == "self" or param.name == "cls" or param.name == "_options":
            continue

        param_info.append(
            _CommandParam(
                param.name,
                param.annotation,
                param.default,
            ),
        )

    return param_info


class _Command:
    def __init__(
        self,
        route: str,
        info: str,
        options: List[CmdOpt],
        callback: Callable,
        params: Optional[List[_CommandParam]] = None,
    ):
        self.route: str = route
        self.info: str = info
        self.options: List[CmdOpt] = options
        self._callback: Callable = callback
        self.params: List[_CommandParam] = params if params else []

    def format(self, short: bool = False) -> str:
        """获取命令的格式化字符串

        Returns:
            str: 命令的格式化字符串
        """

        res = f"{self.route.replace('/', ' ')} "
        res += " ".join([option.gen_format(short) for option in self.options]) + " "
        res += " ".join([param.gen_format() for param in self.params])
        return res

    def desc(self) -> str:
        """获取命令的描述信息

        Returns:
            str: 命令的描述信息
        """

        res = f"Command: {self.route}\n"
        res += f"  Format: {self.format()} \n"
        res += f"  Description: {self.info}\n"
        res += "  Options: \n"
        for option in self.options:
            res += f"    --{option.full_name} (-{option.short_name})\n"
            res += f"        {option.info}\n"
        return res

    def short_desc(self) -> str:
        """获取命令的简短描述信息

        Returns:
            str: 命令的简短描述信息
        """

        res = f"Command: {self.format(short=True)}\n"
        res += f"  Description: {self.info}\n"
        return res

    def run(self, *args, **kwargs) -> Any:
        return self._callback(*args, **kwargs)

    def __str__(self) -> str:
        return f"<Command {self.route}>"

    def __repr__(self) -> str:
        return self.__str__()

    def __eq__(self, o: object) -> bool:
        if not isinstance(o, _Command):
            return False
        return self.route == o.route


class CommandMaster:
    """命令行解析器

    用于解析命令行参数并执行对应的命令

    Examples:
        >>> master = CommandMaster()
        >>> master.reg_option("page", "p", "Show page number.", int, 1)
        >>> @master.reg_command("help", "Show help info.")
        ... def help(_options, command_name: str = "help"):
        ...     print(_options) # {'page': 1}
        ...     print(f"Show help info of command '{command_name}'. (page: {_options['page']})")
        ...     return "Help Info"
        >>> res = master.exec("help test -p 1")   # Show help info of command 'test'. (page: 1)
        >>> print(res)    # Help Info
    """

    def __init__(self) -> None:
        self._options: Dict[str, CmdOpt] = {}
        self._options_short: Dict[str, CmdOpt] = {}

        self._commands: Dict[str, _Command] = {}

    def reg_option(
        self,
        full_name: str,
        short_name: str,
        info: str = "No help info.",
        Type: Union[Type[bool], Type[str], Type[int], Type[float]] = bool,
        default: Any = None,
    ):
        """注册一个命令行选项

        Args:
            full_name (str): 选项的全名 (e.g. help)
            short_name (str): 选项的简称 (e.g. h)
            info (str, optional): 选项的帮助信息. Defaults to "No help info.".
            Type (Union[Type[bool], Type[str], Type[int], Type[float]], optional): 选项的类型. Defaults to bool.
            default (Any, optional): 选项的默认值. Defaults to None.
        """

        option = CmdOpt(
            full_name,
            short_name,
            info,
            Type,
            default,
        )

        if option.full_name in self._options:
            raise ValueError(
                f"The full name of option '{option.full_name}' has been registered.",
            )
        if option.short_name in self._options_short:
            raise ValueError(
                f"The short name of option '{option.short_name}' has been registered.",
            )

        self._options[option.full_name] = option
        self._options_short[option.short_name] = option

        return option

    def reg_command(
        self,
        route: str,
        info: str = "No help info.",
        options: Optional[List[Union[CmdOpt, str]]] = None,
    ) -> Callable[[Callable], Callable]:
        """注册一个命令

        Args:
            route (str): 命令路由 以 / 分割 (e.g. help/test)
            info (str, optional): 命令的帮助信息. Defaults to "No help info.".
            options (Optional[List[Union[CmdOpt, str]]], optional): 命令的选项列表. Defaults to None.

        Returns:
            Callable[[Callable], Callable]: 命令装饰器
        """

        options = options if options else []
        for idx, option in enumerate(options):
            if isinstance(option, str):
                option_obj = self._options.get(f"{option}")
                if option_obj is None:
                    raise ValueError(
                        f"The option '{option}' has not been registered.",
                    )
                options[idx] = option_obj
            elif isinstance(option, CmdOpt) and option.full_name not in self._options:
                raise ValueError(
                    f"The option '{option.full_name}' has not been registered.",
                )

        def command_decorator(func) -> Callable:
            nonlocal options
            if route in self._commands:
                raise ValueError(
                    f"The route '{route}' has been registered.",
                )

            if options is None:
                options = []

            params = _get_func_param_info(func)

            self._commands[route] = _Command(
                route,
                info,
                options,  # type: ignore
                func,
                params,
            )

            # 根据路由段长度降序排序命令便于匹配
            self._commands = dict(
                sorted(
                    self._commands.items(),
                    key=lambda x: len(x[0].split("/")),
                    reverse=True,
                ),
            )

            return func

        return command_decorator

    def exec(self, command_text: str) -> Any:
        """命令解析器 从命令文本中解析出命令对象信息

        Args:
            command_text (str): 命令文本

        Returns:
            Any: 命令执行结果

        Raises:
            ValueError: 命令解析失败
        """

        # 解析命令
        command_split = advance_split(command_text, " ", True)

        for route, command in self._commands.items():
            route_split = advance_split(route, "/")

            # 匹配命令路由
            if len(route_split) > len(command_split) or route != "/".join(
                command_split[: len(route_split)],
            ):
                continue

            # 获取到命令对象后开始解析选项
            options: Dict[str, Any] = {}
            params: Dict[str, Any] = {}
            params_tmp_list: List[str] = []

            # 先填充默认值
            for option in command.options:
                options[option.full_name] = option.default

            command_split = command_split[len(route_split) :]

            # 解析选项
            i = 0
            while i < len(command_split):
                option: CmdOpt
                for opt in self._options.values():
                    if opt.match(command_split[i]):
                        option = opt
                        break
                else:
                    params_tmp_list.append(command_split[i])
                    i += 1
                    continue

                if option.has_value():
                    if i + 1 >= len(command_split):
                        raise ValueError(
                            f"Option '{option.full_name}' needs a value.",
                        )
                    options[option.full_name] = option.Type(
                        command_split[i + 1],
                    )
                    i += 1
                else:
                    options[option.full_name] = True
                i += 1

            # 解析参数
            for i in range(len(command.params)):
                if i >= len(params_tmp_list):
                    break
                params[command.params[i].name] = command.params[i].Type(
                    params_tmp_list[i],
                )

            # 检查是否有多余的参数
            if len(params_tmp_list) > len(command.params):
                raise ValueError(
                    f"Too many parameters for command '{command.route}'.",
                )

            return command.run(_options=options, **params)

        else:
            raise ValueError(f'Command "{command_text[0]}" Not Found.')

    def gen_short_desc(self) -> str:
        """生成所有命令的简短描述信息

        Returns:
            str: 所有命令的简短描述信息
        """

        res = ""
        for command in self._commands.values():
            res += command.short_desc()
        return res

    def gen_desc(self) -> str:
        """生成所有命令的描述信息

        Returns:
            str: 所有命令的描述信息
        """

        res = ""
        for command in self._commands.values():
            res += command.desc()
        return res



from typing import Dict, Optional, Tuple, cast

import aiohttp
import requests


def stringfy_cookies(cookies: Dict) -> str:
    """将cookies转换为字符串"""
    return "; ".join([f"{k}={v}" for k, v in cookies.items()])


def parse_cookies(cookies: str) -> Dict:
    """将字符串转换为cookies"""
    return dict(item.split("=") for item in cookies.split(";"))


def fetch(
    url,
    method: str = "get",
    params: Optional[Dict] = None,
    data: str = "",
    headers: Optional[Dict] = None,
    cookies: Optional[Dict] = None,
    proxy_server: str = "",
    timeout: int = 60,
    ssl_verify: bool = True,
    allow_redirects=True,
) -> Tuple[int, str, requests.Response]:
    """发起同步请求

    Args:
        url (str): 请求地址
        method (str, optional): 请求方法. Defaults to "get".
        params (Optional[Dict], optional): 请求参数. Defaults to None.
        data (str, optional): 请求数据. Defaults to "".
        headers (Optional[Dict], optional): 请求头. Defaults to None.
        cookies (Optional[Dict], optional): 请求cookies. Defaults to None.
        proxy_server (str, optional): 代理服务器. Defaults to "".
        timeout (int, optional): 请求超时时间. Defaults to 60.
        ssl_verify (bool, optional): 是否验证ssl. Defaults to True.
        allow_redirects (bool, optional): 是否允许重定向. Defaults to True.
    Returns:
        Tuple[int, str, requests.Response]: [description]
        status_code: 状态码
        text: 响应文本
        raw_resp: 原始响应对象
    """

    if headers is None:
        headers = {}
    if params is None:
        params = {}

    if proxy_server:
        proxies = {
            "http": f"http://{proxy_server}",
            "https": f"https://{proxy_server}",
        }
    else:
        proxies = None

    resp: requests.Response = cast(
        requests.Response,
        getattr(requests, method.lower())(
            url,
            params=params,
            data=data,
            headers=headers,
            cookies=cookies,
            proxies=proxies,
            timeout=timeout,
            verify=ssl_verify,
            allow_redirects=allow_redirects,
        ),
    )
    return resp.status_code, resp.text, resp


async def async_fetch(
    url,
    method: str = "get",
    params: Optional[Dict] = None,
    data: str = "",
    headers: Optional[Dict] = None,
    cookies: Optional[Dict] = None,
    proxy_server: str = "",
    timeout: int = 60,
    ssl_verify: bool = True,
    allow_redirects: bool = True,
) -> Tuple[int, str, aiohttp.ClientResponse]:
    if headers is None:
        headers = {}
    if cookies:
        headers["Cookie"] = stringfy_cookies(cookies)
    if params is None:
        params = {}

    # 使用 TCPConnector 来处理代理和SSL验证
    conn = aiohttp.TCPConnector(
        verify_ssl=ssl_verify,
    )

    # 创建 ClientSession 实例时指定 connector 和 headers
    async with aiohttp.ClientSession(
        connector=conn if proxy_server else None,
        headers=headers,
        timeout=aiohttp.ClientTimeout(total=timeout),
    ) as session, getattr(session, method.lower())(
        url,
        params=params,
        data=data,
        allow_redirects=allow_redirects,
        proxy=proxy_server if proxy_server else None,
        ssl=ssl_verify,
    ) as resp:
        resp = cast(aiohttp.ClientResponse, resp)
        return resp.status, await resp.text(), resp



from typing import List

import pkg_resources


def get_pkg_version(pkg_name: str) -> str:
    """获取包的版本"""
    return pkg_resources.get_distribution(pkg_name).version


class Version:

    def __init__(self, version: str):
        self.version = version
        self.major, self.minor, self.patch = [int(x) for x in version.split(".")]

    def __lt__(self, other: "Version") -> bool:
        return self.__compare_versions(self.version, other.version) < 0

    def __le__(self, other: "Version") -> bool:
        return self.__compare_versions(self.version, other.version) <= 0

    def __eq__(self, other: "Version") -> bool:
        return self.__compare_versions(self.version, other.version) == 0

    def __ne__(self, other: "Version") -> bool:
        return self.__compare_versions(self.version, other.version) != 0

    def __gt__(self, other: "Version") -> bool:
        return self.__compare_versions(self.version, other.version) > 0

    def __ge__(self, other: "Version") -> bool:
        return self.__compare_versions(self.version, other.version) >= 0

    def __str__(self) -> str:
        return self.version

    def __repr__(self) -> str:
        return self.version

    @classmethod
    def __compare_versions(cls, v1: str, v2: str) -> int:
        """
        Compare two version strings and return -1 if v1 < v2, 0 if v1 == v2, and 1 if v1 > v2.
        """

        def normalize(v: str) -> List[int]:
            return [int(x) for x in v.split(".")]

        v1n: List[int] = normalize(v1)
        v2n: List[int] = normalize(v2)

        if v1n < v2n:
            return -1
        if v1n > v2n:
            return 1
        return 0

